﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Services.Hive.Common;
using HeuristicLab.Clients.Hive.Slave;
using HeuristicLab.Services.Hive.Common.DataTransfer;
using HeuristicLab.Clients.Hive.Slave.Properties;


namespace HeuristicLab.Clients.Hive.Salve {
  /// <summary>
  /// accesses the Server and sends his data (uuid, uptimes, hardware config) 
  /// </summary>
  public class ConfigManager {
    private static ConfigManager instance = null;
    public static ConfigManager Instance {
      get {
        if (instance == null) {
          instance = new ConfigManager();
        }
        return instance;
      }
    }

    public Core Core { get; set; }
    private HeuristicLab.Services.Hive.Common.DataTransfer.Slave hardwareInfo;

    /// <summary>
    /// Constructor for the singleton, must recover Guid, Calendar, ...
    /// </summary>
    private ConfigManager() {
      hardwareInfo = new HeuristicLab.Services.Hive.Common.DataTransfer.Slave();

      if (Settings.Default.Guid == Guid.Empty) {
        hardwareInfo.Id = Guid.NewGuid();
        Settings.Default.Guid = hardwareInfo.Id;
        Settings.Default.Save();
      } else
        hardwareInfo.Id = Settings.Default.Guid;

      hardwareInfo.Cores = Environment.ProcessorCount;
      hardwareInfo.Memory = 1024;
      hardwareInfo.Name = Environment.MachineName;
    }

    /// <summary>
    /// Get all the Information about the client
    /// </summary>
    /// <returns>the ClientInfo object</returns>
    public HeuristicLab.Services.Hive.Common.DataTransfer.Slave GetClientInfo() {
      //TODO: how to display connectedsince in gui?
      //hardwareInfo.Login = WcfService.Instance.ConnectedSince;
      return hardwareInfo;
    }

    /// <summary>
    /// collects and returns information that get displayed by the Client Console
    /// </summary>
    /// <returns></returns>
    public StatusCommons GetStatusForClientConsole() {
      //Todo: Locking
      StatusCommons st = new StatusCommons();
      st.ClientGuid = hardwareInfo.Id;

      st.Status = WcfService.Instance.ConnState;
      st.ConnectedSince = WcfService.Instance.ConnectedSince;

      st.TotalCores = hardwareInfo.Cores.HasValue ? hardwareInfo.Cores.Value : 0;
      st.FreeCores = hardwareInfo.Cores.HasValue ? hardwareInfo.Cores.Value - GetUsedCores() : 0;

      st.JobsAborted = SlaveStatusInfo.JobsAborted;
      st.JobsDone = SlaveStatusInfo.JobsProcessed;
      st.JobsFetched = SlaveStatusInfo.JobsFetched;

      Dictionary<Guid, Executor> engines = Core.ExecutionEngines;
      st.Jobs = new List<JobStatus>();

      lock (engines) {
        foreach (KeyValuePair<Guid, Executor> kvp in engines) {
          Executor e = kvp.Value;
          st.Jobs.Add(new JobStatus { JobId = e.JobId, ExecutionTime = e.ExecutionTime, Since = e.CreationTime });
        }
      }
      return st;
    }

    public Dictionary<Guid, TimeSpan> GetExecutionTimeOfAllJobs() {
      Dictionary<Guid, TimeSpan> prog = new Dictionary<Guid, TimeSpan>();
      Dictionary<Guid, Executor> engines = Core.ExecutionEngines;
      lock (engines) {
        foreach (KeyValuePair<Guid, Executor> kvp in engines) {
          Executor e = kvp.Value;
          prog[e.JobId] = e.ExecutionTime;
        }
      }
      return prog;
    }

    public int GetUsedCores() {
      Dictionary<Guid, Executor> engines = Core.ExecutionEngines;
      Dictionary<Guid, Job> jobs = Core.Jobs;
      int usedCores = 0;
      lock (engines) {
        foreach (KeyValuePair<Guid, Job> kvp in jobs)
          usedCores += kvp.Value.CoresNeeded;
      }
      return usedCores;
    }

  }
}
